% Users must cite two following papers:
% 1) Kim, Shaver, and Funk. Forthcoming. "From Mass to Motion: Conceptualizing and Measuring the Dynamics of Industry Clusters," Strategic Management Journal
% 2) Bai, Jushan and Pierre Perron. 2003. "Computation and Analysis of Multiple Structural Change Models," Journal of Applied Econometrics, 18, 1-22.

clear;
clc; 
close all  %Close all the open window including figures

diary BP_log.txt
%Directory containing the sub-files.
addpath('supp_files')

name = 'Clevel_output_BPtest_input_semiconductor'; % Input file #1: zscores of the focal industry establishments (or employees) in a year-region
[data header] = xlsread([name,'.xlsx']);

year = data(:,1);
date_col = cellstr(num2str(year));
for i = 1:length(year)
    date_col(i) = cellstr(['yr',char(date_col(i))]);
end

data(:,1) = [];

[obs,numvar] = size(data);  % This is for measuring(saving) size


start_year = min(year);
end_year = max(year);
start_date=1;
end_date=end_year-start_year+1;

ID = header(1,2:end);
lazy_matrix = [0;(start_year:1:end_year)'];
size(data,2)
for M = 1:size(data,2)

    M

dates=date_col(start_date:end_date,1);  
y=data(start_date:end_date,M); 

% (non)prespecified # of breaks
fixn=0;                    % value should be assigned by the user. (0 refers to non-prespecified)
if (fixn>0)
    estimfix=1;            % set to 1 if want to estimate the model with a prespecified number
else                       % of breaks equal to fixn set below
    estimfix=0;
end
                           % set up the data, y is the dependent variable. 
                           % "z" is the matrix of regressors (bigt,q), 
                           %      whose coefficients are allowed to change. 
                           % "x" is a (bigt,p) matrix of regressors 
                           %      with coefficients fixed across regimes. 
                           % Note: initialize x to [] if p=0.

bigt=length(y);            % set effective sample size
z=[[1:bigt]' ones(bigt,1)];% 2 regressors are defined here; 
                           % ones=constant, so regressor for "a(alpha) coefficient" is one. 
                           % "ones" is a function, resulting the column format
                           % " [1:bigt]' " is a regressor for "b(beta) coefficient"
                           % Coz this is not a function, I have to transpose by putting '. 
x=[]; 
q=2;                      % number of regressors z
p=0;                      % number of regressors x 

m=5*(1-estimfix)+fixn*estimfix; % maximum number of structural changes allowed
                                % Users can change the number by changing "the
                                % first number" (e.g. Want "5" maximum ->
                                % m="5"*(1-estimfix)+fixn*estimfix

% Minimal segment size                                                                        
eps1=.15;                 % value of the trimming (in percentage) for the construction 
                          % and critical values of the supF type tests (used in the 
                          % supF test, the Dmax, the supF(l+1|l) and the sequential 
                          % procedure). If these tests are used, h below should be set 
                          % at int(eps1*bigt). But if the tests are not required, 
                          % estimation can be done with an arbitrary h. 
                          % (from the author of code-i.e., considering their sample size; 
                          % There are fiveoptions: eps1= .05, .10, .15, .20, or .25. 
                          % For each option, the maximal value of m above is: 10 for eps=.05, 
                          % 8 for eps1=.10, 5 for eps1=.15, 3 for eps1=.20, and 2 for eps1=.25).

h=round(eps1*bigt);       % minimal length of a segment (h >= q). Note: if robust=1, h 
                          % should be set at a larger value. 

%%
% The followings are options if p > 0 -------------------------------                          
fixb=0;                   % set to 1 if use fixed initial values for beta
betaini=0;                % if fixb=1, load the initial value of beta
maxi=20;                  % maximum number of iterations for the nonlinear procedure to 
                          % obtain global minimizers
printd=1;                 % set to 1 if want the output from the iterations to be printed
eps=0.0001;               % criterion for the convergence
% --------------------------------------------------------------------

robust=0;                % set to 1 if want to allow for heterogeneity and autocorrelation 
                         % in the residuals, 0 otherwise. The method used is Andrews(1991) 
                         % automatic bandwidth with AR(1) approximation and the quadratic 
                         % kernel. Note: Do not set to 1 if lagged dependent variables are 
                         % included as regressors.
prewhit=1;               % set to 1 if want to apply AR(1) prewhitening prior to estimating 
                         % the long run covariance matrix.
hetdat=1;                % option for the construction of the F tests. Set to 1 if want to
                         % allow different moment matrices of the regressors across segments. 
                         % If hetdat=0, the same moment matrices are assumed for each segment 
                         % and estimated from the ful sample. It is recommended to set 
                         % hetdat=1 if p>0.
hetvar=1;                % option for the construction of the F tests.Set to 1 if want to allow 
                         % for the variance of the residuals to be different across segments. 
                         % If hetvar=0, the variance of the residuals is assumed constant 
                         % across segments and constructed from the ful sample. This option 
                         % is not available when robust=1.  
hetomega=1;              % used in the construction of the confidence intervals for the break 
                         % dates. If hetomega=0, the long run covariance matrix of zu is 
                         % assumed identical across segments (the variance of the errors u 
                         % if robust=0)
hetq=1;                  % used in the construction of the confidence intervals for the break 
                         % dates. If hetq=0, the moment matrix of the data is assumed identical 
                         % across segments.
doglobal=1;              % set to 1 if want to cal the procedure to obtain global minimizers
dotest=1;                % set to 1 if want to construct the supF, UDmax and WDmax tests 
                         % doglobal must be set to 1 to run this procedure.
dospflp1=1;              % set to 1 if want to construct the supF(l+1|l) tests where under
                         % the null the l breaks are obtained using global minimizers. 
                         % doglobal must be set to 1 to run this procedure.
doorder=1;               % set to 1 if want to cal the procedure that selects the number of
                         % breaks using information criteria. doglobal must be set to 1 to 
                         % run this procedure.
dosequa=1;               % set to 1 if want to estimate the breaks sequentialy and estimate 
                         % the number of breaks using supF(l+1|l) test   
dorepart=1;              % set to 1 if want to modify the brforeak dates obtained from the 
                         % sequential method using the repartition method of Bai (1995),
                         % Estimating breaks one at a time. This is needed for the confidence 
                         % intervals obtained with estim below to be valid.
estimbic=0;              % set to 1 if want to estimate the model with the number of breaks 
                         % selected by BIC.
estimlwz=0;              % set to 1 if want to estimate the model with the number of breaks  
                         % selected by LWZ
estimseq=1;              % set to 1 if want to estimate the model with the number of breaks
                         % selected using the sequential procedure
estimrep=0;              % set to 1 if want to estimate the model with the breaks selected
                         % using the repartition method

%%
[thres coeff daty]=pbreak(bigt,y,z,q,m,h,eps1,robust,prewhit,hetomega,hetq,doglobal,dotest,dospflp1,doorder,dosequa,dorepart,estimbic,estimlwz,estimseq,estimrep,estimfix,fixb,x,p,eps,maxi,betaini,printd,hetdat,hetvar,fixn,dates);
%daty=103;
%create a graph
indy=[1:bigt]';
crisis_obs=length(data)-end_date;


% Parameter for assigning Regression coefficient at the break point to 
% that of the previous segment (*Set to 1 is the user want the regression
% coefficient of the previous segment to be assigned at the break point
% year. 0 otherwise)

coeff_at_break = 0;
fileID = fopen('regression_output_semiconductor_est.csv','a+');
breaks = daty;
M
if breaks(1) > 0
  reg_coeff = zeros(1,length(breaks));
  for k=0:length(breaks)
    if k == 0
        xtemp = 1:breaks(1);
        ytemp = y(1:breaks(1))';
    elseif k == length(breaks)
        xtemp = breaks(end):end_date;
        ytemp = y(breaks(end):end)';
    else
        xtemp = breaks(k):breaks(k+1);
        ytemp = y(breaks(k):breaks(k+1))';
    end

    regout = fitlm(xtemp,ytemp);
    regout_str = evalc('disp(regout)');
    filtered = strsplit(regout_str,'\n'); filtered = filtered(:,8);
    filtered = strsplit(char(filtered),' ');
    coeff_ = filtered(4);
    reg_coeff(k+1) = str2double(coeff_);
  end
  to_print = [(start_year:1:start_year+end_date-1)' ones(1,end_date)'*reg_coeff(1)];

  if coeff_at_break
    for iter = 1:length(breaks)-1
        to_print(breaks(iter)+1:breaks(iter+1)+1,2) = reg_coeff(iter+1);
    end
    to_print(breaks(end)+1:end,2) = reg_coeff(end);
  else
    for iter = 1:length(breaks)-1
        to_print(breaks(iter)+1:breaks(iter+1)-0,2) = reg_coeff(iter+1);
    end
    to_print(breaks(end)+1:end,2) = reg_coeff(end);
  end

else
  xtemp = 1:end_date;
  ytemp = y(1:end);
  regout = fitlm(xtemp,ytemp);
  regout_str = evalc('disp(regout)');
  filtered = strsplit(regout_str,'\n'); filtered = filtered(:,8);
  filtered = strsplit(char(filtered),' ');
  coeff_ = filtered(4);
  reg_coeff(1) = str2double(coeff_);
  to_print = [(start_year:1:start_year+end_date-1)' ones(1,end_date)'*reg_coeff(1)];
end

name = cell2mat(ID(M));
name = name(end-4:end);
%fprintf(fileID,'%s\n',descriptor);
%for kk = 1:length(to_print)
%    fprintf(fileID,'%d,%f\n',to_print(kk,1),to_print(kk,2));
%end
    lazy_matrix = [lazy_matrix(1:end_date+1,:) [str2num(name);to_print(:,2)]];
end
outmat = [header(1,:); num2cell(lazy_matrix(2:end,:))];
% xlswrite('cluster_dynamics_output.xlsx',outmat)
writetable(table(outmat),'regression_output_semiconductor_est.xlsx','WriteVariableNames',false)

diary off